// Counter/ReverseCounter server for omniORB 3.0
//   HORIKAWA Hisashi

#include <cstdio>
#include <cctype>
#include <iostream>
#include "counter.hh"

using namespace std;

void do_test(Counter_ptr counter)
{
    CORBA::String_var name = counter->name();
    printf("name = %s\n", name.in());

    int i;
    printf("incr(): ");
    for (i = 0; i < 5; i++) {
        counter->incr();
        printf(" %d", counter->value());
    }
    printf(".\n");

    printf("decr(): ");
    for (i = 0; i < 3; i++) {
        counter->decr();
        printf(" %d", counter->value());
    }
    printf(".\n");
}

char* read_line(FILE* fp)
    // CRLF
{
    static char buf[1000];

    fgets(buf, sizeof(buf), fp);
    int len;
    while ((len = strlen(buf)) > 0) {
        if (isspace(buf[len - 1]))
            buf[len - 1] = '\0';
        else
            break;
    }

    return buf;
}

const char* ior_file = "../counter.ior";

int main(int argc, char* argv[])
{
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");
            // omniORB3άǤʤ

    try {
        FILE* fp = fopen(ior_file, "r");
        if (!fp) {
            perror("file open");
            return 1;
        }
        CORBA::Object_var obj_nc = orb->string_to_object(read_line(fp));
        CORBA::Object_var obj_rc = orb->string_to_object(read_line(fp));
        fclose(fp);
        if (!obj_nc || !obj_rc) {
            printf("ior error.\n");
            return 1;
        }

        Counter_ptr nc = Counter::_narrow(obj_nc);
        Counter_ptr rc = ReverseCounter::_narrow(obj_rc);
        if (CORBA::is_nil(nc) || CORBA::is_nil(rc)) {
            printf("cannot invoke on a nil object\n");
            return 1;
        }

        do_test(nc);
        do_test(rc);
    }
    catch (CORBA::SystemException&) {
        cerr << "Caught CORBA::SystemException." << endl;
    }
    catch (CORBA::Exception& e) {
        cerr << "Caught CORBA::Exception." << endl;
    }
    catch (...) {
        printf("unknown exception.\n");
    }

    return 0;
}
