// Counter/ReverseCounter server for omniORB 3.0
//   HORIKAWA Hisashi

#include <cstdio>
#include <iostream>
#include "counter.hh"

using namespace std;

////////////////////////////////////////////////////////////////////////
// CounterImpl

class CounterImpl: public virtual POA_Counter,
                   public virtual PortableServer::RefCountServantBase
{
protected:
    short count_;
public:
    CounterImpl(): count_(0) { }
    virtual ~CounterImpl() { }

    virtual CORBA::Short value(){ return count_; }
    virtual void incr() { count_++; }
    virtual void decr() { count_--; }
    virtual char* name() { return CORBA::string_dup("Counter"); }
protected:
    CounterImpl(short v): count_(v) { }
};

////////////////////////////////////////////////////////////////////////
// ReverseCounterImpl

class ReverseCounterImpl: public virtual POA_ReverseCounter,
                          public virtual PortableServer::RefCountServantBase,
                          public virtual CounterImpl
{
    typedef CounterImpl super;
public:
    ReverseCounterImpl(): super(1000) { }
    virtual ~ReverseCounterImpl() { }

    virtual void incr() { count_--; }
    virtual void decr() { count_++; }
    virtual char* name() { return CORBA::string_dup("ReverseCounter"); }
};

////////////////////////////////////////////////////////////////////////

static const char* ior_file = "../counter.ior";

int main(int argc, char* argv[])
{
    try {
        // Initialise the ORB.
        CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

        // Obtain a reference to the root POA.
        CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);

        // We allocate the objects on the heap.  Since these are reference
        // counted objects, they will be deleted by the POA when they are no
        // longer needed.
        CounterImpl* myCounter = new CounterImpl();
        ReverseCounterImpl* myReverseCounter = new ReverseCounterImpl();
    
        // Activate the objects.  This tells the POA that the objects are
        // ready to accept requests.
        PortableServer::ObjectId_var myCounter_iid
            = poa->activate_object(myCounter);
        PortableServer::ObjectId_var myReverseCounter_iid
            = poa->activate_object(myReverseCounter);
    
        // Obtain a reference to each object and output the stringified
        // IOR to ior-file
        FILE* fp = fopen(ior_file, "w");
        if (!fp) {
            perror("ior-file open failed");
            return 1;
        }
        CORBA::String_var sior;
        sior = orb->object_to_string(myCounter->_this());
        fprintf(fp, "%s\n", (const char*) sior);
        sior = orb->object_to_string(myReverseCounter->_this());
        fprintf(fp, "%s\n", (const char*) sior);
        fclose(fp);

        // Obtain a POAManager, and tell the POA to start accepting
        // requests on its objects.
        PortableServer::POAManager_var poaMgr = poa->the_POAManager();
        poaMgr->activate();

        // main loop
        printf("ready.\n");
        orb->run();
        
        poa->destroy(true, true);
        delete myCounter;
        delete myReverseCounter;
        remove(ior_file);
    }
    catch(CORBA::SystemException&) {
        cerr << "Caught CORBA::SystemException." << endl;
    }
    catch(CORBA::Exception&) {
        cerr << "Caught CORBA::Exception." << endl;
    }
    catch(omniORB::fatalException& fe) {
        cerr << "Caught omniORB::fatalException:" << endl;
        cerr << "  file: " << fe.file() << endl;
        cerr << "  line: " << fe.line() << endl;
        cerr << "  mesg: " << fe.errmsg() << endl;
    }
    catch(...) {
        cerr << "Caught unknown exception." << endl;
    }

    return 0;
}

