// CORBAץ
// client for ORBit

#include "../config.h"

#include <stdio.h>
#include <assert.h>
#include <orb/orbit.h>

#ifdef USE_NAME_SERVICE
  #include <ORBitservices/CosNaming.h>
#elif defined(USE_OAF)
  #include <liboaf/liboaf.h>
#endif

#include "../misc.h"

#include "counter.h"
    // orbit-idlˤä줿.hե
    // counter.idl -> counter.h

//////////////////////////////////////////////////////////////////////

CORBA_Environment ev;

void testCounter(Counter counter)
{
    printf("name = %s\n", Counter_name(counter, &ev));

    int i;
    printf("incr(): ");
    for (i = 0; i < 10; i++) {
        Counter_incr(counter, &ev);
        printf(" %d", Counter__get_value(counter, &ev));
    }
    printf(".\n");

    printf("decr(): ");
    for (i = 0; i < 3; i++) {
        Counter_decr(counter, &ev);
        printf(" %d", Counter__get_value(counter, &ev));
    }
    printf(".\n");
}

void catchException(CORBA_Environment& ev)
{
    if (ev._major != CORBA_NO_EXCEPTION) {
        printf("CORBA exception: %s\n", CORBA_exception_id(&ev));
        CORBA_exception_free(&ev);
        exit(1);
    }
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    CORBA_exception_init(&ev);

#ifdef USE_OAF
    CORBA_ORB orb = oaf_init(argc, argv);
#else
    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    catchException(ev);
#endif

    Counter counter = NULL;
    ReverseCounter rc = NULL;

#ifdef USE_NAME_SERVICE
    CosNaming_NamingContext naming;
#if 0
    // ޥɥ饤ǥ͡ॵСIORͿ
    naming = CORBA_ORB_resolve_initial_references(orb, "NameService", &ev);
#else
    // 2000.11.01 ORBitstring_to_object()ǤǤ
    FILE* fp = fopen("../naming.ior", "r");
    assert(fp);
    naming = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    fclose(fp);
#endif
    catchException(ev);
    if (!naming) {
        printf("resolve NameService failed.\n");
        return 1;
    }

    CosNaming_Name* name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(COUNTER_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    counter = CosNaming_NamingContext_resolve(naming, name, &ev);

    name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(REVERSE_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    rc = CosNaming_NamingContext_resolve(naming, name, &ev);
#elif defined(USE_OAF)
    counter = oaf_activate_from_id(NC_IID, 0, NULL, &ev);
    catchException(ev);
    if (CORBA_Object_is_nil(counter, &ev)) {
        printf("activation '%s' failed.\n", NC_IID);
        exit(1);
    }
    rc = oaf_activate_from_id(RC_IID, 0, NULL, &ev);
    catchException(ev);
    if (CORBA_Object_is_nil(rc, &ev)) {
        printf("activation '%s' failed.\n", RC_IID);
        exit(1);
    }
/*
    counter = oaf_activate("repo_ids.has('IDL:Counter:1.0')", NULL, 0, NULL, &ev);
    catchException(ev);
    rc = oaf_activate("repo_ids.has('IDL:ReverseCounter:1.0')", NULL, 0, NULL, &ev);
    catchException(ev);
*/
#else
    FILE* fp = fopen(ior_file, "r");
    if (!fp) {
        perror("ior-file open");
        return 1;
    }
    counter = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    catchException(ev);
    rc = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    catchException(ev);
    fclose(fp);
#endif // USE_NAME_SERVICE

    if (!counter || !rc) {
        printf("ior error.\n");
        return 1;
    }

    testCounter(counter);
    testCounter(rc);

    CORBA_Object_release(counter, &ev);
    CORBA_Object_release(rc, &ev);

    return 0;
}
