// server for orbit

#include "../config.h"

#include <stdio.h>
#include <orb/orbit.h>

#ifdef USE_NAME_SERVICE
  #include <ORBitservices/CosNaming.h>
#elif defined(USE_OAF)
  #include <liboaf/liboaf.h>
#endif

#include "counter.h"

////////////////////////////////////////////////////////////////////////
// CounterImpl

struct CounterData {
    short value;
    CounterData(short v_ = 0): value(v_) { }
};

#define Counter_DATA(x) ((CounterData*) ((POA_Counter*) (x))->vepv->Counter_epv->_private)

CORBA_short CounterImpl_get_value(PortableServer_Servant servant,
                                  CORBA_Environment* ev)
{
    return Counter_DATA(servant)->value;
}

void CounterImpl_incr(PortableServer_Servant servant, CORBA_Environment* ev)
{
    Counter_DATA(servant)->value++;
}

void CounterImpl_decr(PortableServer_Servant servant, CORBA_Environment* ev)
{
    Counter_DATA(servant)->value--;
}

CORBA_char* CounterImpl_name(PortableServer_Servant servant,
                             CORBA_Environment* ev)
{
    return CORBA_string_dup("Counter");
}

POA_Counter* CounterImpl_new(CORBA_Environment* ev)
{
    PortableServer_ServantBase__epv super_epv = { NULL, NULL, NULL };
    POA_Counter__epv epv = {
        NULL, // Counter饹protectedǡǼ
        CounterImpl_get_value,
        CounterImpl_incr,
        CounterImpl_decr,
        CounterImpl_name };

    POA_Counter* servant = new POA_Counter();
    servant->_private = NULL; // ORB
    servant->vepv = new POA_Counter__vepv();
    servant->vepv->_base_epv = new PortableServer_ServantBase__epv();
    servant->vepv->Counter_epv = new POA_Counter__epv();

    *servant->vepv->_base_epv = super_epv;
    *servant->vepv->Counter_epv = epv;

    POA_Counter__init(servant, ev);
    Counter_DATA(servant) = new CounterData();
    return servant;
}

void CounterImpl_delete(POA_Counter* servant, CORBA_Environment* ev)
{
    delete Counter_DATA(servant);
    POA_Counter__fini(servant, ev);

    delete servant->vepv->Counter_epv;
    delete servant->vepv->_base_epv;
    delete servant->vepv;
    delete servant;
}

////////////////////////////////////////////////////////////////////////
// ReverseCounterImpl

void ReverseCounterImpl_incr(PortableServer_Servant servant, CORBA_Environment* ev)
{
    Counter_DATA(servant)->value--;
}

void ReverseCounterImpl_decr(PortableServer_Servant servant, CORBA_Environment* ev)
{
    Counter_DATA(servant)->value++;
}

CORBA_char* ReverseCounterImpl_name(PortableServer_Servant servant,
                                    CORBA_Environment* ev)
{
    return CORBA_string_dup("ReverseCounter");
}

POA_ReverseCounter* ReverseCounterImpl_new(CORBA_Environment* ev)
{
    PortableServer_ServantBase__epv super_super_epv = { NULL, NULL, NULL };
    POA_Counter__epv super_epv = {
        NULL,
        CounterImpl_get_value,
        ReverseCounterImpl_incr,
        ReverseCounterImpl_decr,
        ReverseCounterImpl_name };
    POA_ReverseCounter__epv epv = { NULL };
    
    POA_ReverseCounter* servant = new POA_ReverseCounter();
    servant->_private = NULL; // ORB
    servant->vepv = new POA_ReverseCounter__vepv();
    servant->vepv->_base_epv = new PortableServer_ServantBase__epv();
    servant->vepv->Counter_epv = new POA_Counter__epv();
    servant->vepv->ReverseCounter_epv = new POA_ReverseCounter__epv();

    *servant->vepv->_base_epv = super_super_epv;
    *servant->vepv->Counter_epv = super_epv;
    *servant->vepv->ReverseCounter_epv = epv;

    POA_ReverseCounter__init(servant, ev);
    Counter_DATA(servant) = new CounterData(1000);
    return servant;
}

void ReverseCounterImpl_delete(POA_ReverseCounter* servant, CORBA_Environment* ev)
{
    delete Counter_DATA(servant);
    POA_ReverseCounter__fini(servant, ev);

    delete servant->vepv->ReverseCounter_epv;
    delete servant->vepv->Counter_epv;
    delete servant->vepv->_base_epv;
    delete servant->vepv;
    delete servant;
}

//////////////////////////////////////////////////////////////////////

void catchException(CORBA_Environment& ev)
{
    if (ev._major != CORBA_NO_EXCEPTION) {
        printf("CORBA exception: %s\n", CORBA_exception_id(&ev));
        CORBA_exception_free(&ev);
        exit(1);
    }
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    CORBA_Environment ev;
    CORBA_exception_init(&ev);

#ifdef USE_OAF
    CORBA_ORB orb = oaf_init(argc, argv);
#else
    // Initialise the ORB.
    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    catchException(ev);
#endif

    // Obtain a reference to the root POA.
    PortableServer_POA poa
            = (PortableServer_POA)
            CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
    catchException(ev);

    // We allocate the objects on the heap.  Since these are reference
    // counted objects, they will be deleted by the POA when they are no
    // longer needed.
    POA_Counter* myCounter = CounterImpl_new(&ev);
    catchException(ev);
    POA_ReverseCounter* myReverseCounter = ReverseCounterImpl_new(&ev);
    catchException(ev);

    // Activate the objects.  This tells the POA that the objects are
    // ready to accept requests.
    PortableServer_POA_activate_object(poa, myCounter, &ev);
    catchException(ev);
    PortableServer_POA_activate_object(poa, myReverseCounter, &ev);
    catchException(ev);
/*    
    PortableServer_ObjectId nc_oid = {
        0,  // maximum
        strlen(COUNTER_NAME) + 1,
        (unsigned char*) COUNTER_NAME,
        0   // release
    };
    PortableServer_ObjectId rc_oid = {
        0,  // maximum
        strlen(REVERSE_NAME) + 1,
        (unsigned char*) REVERSE_NAME,
        0   // release
    };

    PortableServer_POA_activate_object_with_id(poa, &nc_oid, nc, &ev);
    catchException(ev);
    PortableServer_POA_activate_object_with_id(poa, &rc_oid, rc, &ev);
    catchException(ev);
*/
#ifdef USE_NAME_SERVICE
    CosNaming_NamingContext naming
            = CORBA_ORB_resolve_initial_references(orb, "NameService", &ev);
    if (!naming) {
        printf("resolve NameService failed.\n");
        return 1;
    }

    CosNaming_Name* name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(COUNTER_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    CosNaming_NamingContext_rebind(naming, name,
            PortableServer_POA_servant_to_reference(poa, nc, &ev), &ev);

    name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(REVERSE_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    CosNaming_NamingContext_rebind(naming, name,
            PortableServer_POA_servant_to_reference(poa, rc, &ev), &ev);
#else
    // Obtain a reference to each object and output the stringified
    // IOR to ior-file
    CORBA_Object_struct* counter_ref
        = PortableServer_POA_servant_to_reference(poa, myCounter, &ev);
    catchException(ev);
    CORBA_Object_struct* reverse_ref
        = PortableServer_POA_servant_to_reference(poa, myReverseCounter, &ev);
    catchException(ev);
    if (!counter_ref || !reverse_ref) {
        printf("cannot get object\n");
        return 1;
    }

#ifdef USE_OAF
    oaf_active_server_register(NC_IID, counter_ref);
    oaf_active_server_register(RC_IID, reverse_ref);
/*
  2000.11.06 Υɤ㳰ȯ롣
  oaf_active_server_register()ؤIIDϤʤ
    oaf_active_server_register("IDL:Counter:1.0", counter_ref);
    oaf_active_server_register("IDL:ReverseCounter:1.0", reverse_ref);
*/
#else
    FILE* fp = fopen(ior_file, "w");
    if (!fp) {
        perror("ior-file open failed");
        return 1;
    }
    char* objstr = CORBA_ORB_object_to_string(orb, counter_ref, &ev);
    fprintf(fp, "%s\n", objstr);
    CORBA_free(objstr);
    objstr = CORBA_ORB_object_to_string(orb, reverse_ref, &ev);
    fprintf(fp, "%s\n", objstr);
    CORBA_free(objstr);
    fclose(fp);
#endif // USE_OAF
#endif // USE_NAME_SERVICE

    // Obtain a POAManager, and tell the POA to start accepting
    // requests on its objects.
    PortableServer_POAManager_activate(
        PortableServer_POA__get_the_POAManager(poa, &ev), &ev);

    // main loop
    printf("ready.\n");
#ifdef USE_OAF
    while (true)
        g_main_iteration(TRUE);
#else
    CORBA_ORB_run(orb, &ev);
#endif

#ifndef USE_NAME_SERVICE
    remove(ior_file);
#endif
    return 0;
}
